//------------------------------------------------------------------
// Stochastic Momentum Index developed by William Blau
//------------------------------------------------------------------
#property copyright "www.forex-tsd.com"
#property link      "www.forex-tsd.com"

#property indicator_separate_window
#property indicator_buffers    4
#property indicator_color1     LimeGreen
#property indicator_color2     PaleVioletRed
#property indicator_color3     DarkSlateGray
#property indicator_color4     Goldenrod
#property indicator_width1     2
#property indicator_width2     2
#property indicator_width3     2
#property indicator_level1     0.0
#property indicator_levelcolor MediumOrchid

//
//
//
//
//

extern string TimeFrame       = "Current time frame";
extern int    SMIPeriod1      = 13;
extern int    SMIPeriod2      = 25;
extern int    SMIPeriod3      =  2;
extern int    SignalPeriod    =  5;
extern bool   ColorOnSlope    = true;

extern bool   alertsOn        = true;
extern bool   alertsOnCurrent = true;
extern bool   alertsMessage   = true;
extern bool   alertsSound     = false;
extern bool   alertsNotify    = false;
extern bool   alertsEmail     = false;
extern string soundFile       = "alert2.wav"; 

//
//
//
//
//

double SMIBuffer[];
double SMIBufferDa[];
double SMIBufferDb[];
double SignalBuffer[];
double colors[];

//
//
//
//
//

string indicatorFileName;
bool   returnBars;
bool   calculateValue;
int    timeFrame;

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

int init()
{
   IndicatorBuffers(5);
   SetIndexBuffer(0,SMIBufferDa); SetIndexStyle(0,DRAW_HISTOGRAM);
   SetIndexBuffer(1,SMIBufferDb); SetIndexStyle(1,DRAW_HISTOGRAM);
   SetIndexBuffer(2,SMIBuffer);
   SetIndexBuffer(3,SignalBuffer);
   SetIndexBuffer(4,colors);
   
      //
      //
      //
      //
      //
      
         indicatorFileName = WindowExpertName();
         calculateValue    = TimeFrame=="calculateValue"; if (calculateValue) { return(0); }
         returnBars        = TimeFrame=="returnBars";     if (returnBars)     { return(0); }
         timeFrame         = stringToTimeFrame(TimeFrame);
      
      //
      //
      //
      //
      //
      
   IndicatorShortName(timeFrameToString(timeFrame)+" SMI ("+SMIPeriod1+","+SMIPeriod2+","+SMIPeriod3+","+SignalPeriod+")");
   return(0);
}

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

double work[][4];
#define _ema00 0
#define _ema01 1
#define _ema10 2
#define _ema11 3

int start()
{
   int i,r,counted_bars=IndicatorCounted();
      if(counted_bars<0) return(-1);
      if(counted_bars>0) counted_bars--;
         int limit = MathMin(Bars-counted_bars,Bars-1);
         if (returnBars) { SMIBuffer[0] = MathMin(limit+1,Bars-1); return(0); }
         

   //
   //
   //
   //
   //
   
   if (calculateValue || timeFrame == Period())
   {
      if (ArrayRange(work,0)!=Bars) ArrayResize(work,Bars);
      double alfaS = 2.0 / (1.0 + SignalPeriod);
      double alfa1 = 2.0 / (1.0 + SMIPeriod2);
      double alfa2 = 2.0 / (1.0 + SMIPeriod3);
      for (i=limit, r=Bars-i-1; i>=0; i--,r++)
      {
         double highest = High[iHighest(NULL,0,MODE_HIGH,SMIPeriod1,i)];
         double lowest  = Low [iLowest (NULL,0,MODE_LOW ,SMIPeriod1,i)];
         double sm      = Close[i]-(highest+lowest)*0.5;
         double hl      =           highest-lowest;

            work[r][_ema00] = work[r-1][_ema00]+alfa1*(sm             -work[r-1][_ema00]);
            work[r][_ema01] = work[r-1][_ema01]+alfa2*(work[r][_ema00]-work[r-1][_ema01]);
            work[r][_ema10] = work[r-1][_ema10]+alfa1*(hl             -work[r-1][_ema10]);
            work[r][_ema11] = work[r-1][_ema11]+alfa2*(work[r][_ema10]-work[r-1][_ema11]);
            if (work[r][_ema11] != 0)
                  SMIBuffer[i] = 100.00*(work[r][_ema01]/(0.5*work[r][_ema11]));
            else  SMIBuffer[i] = 0;
            SignalBuffer[i] = SignalBuffer[i+1] + alfaS*(SMIBuffer[i]-SignalBuffer[i+1]);
            
            //
            //
            //
            //
            //
            
            SMIBufferDa[i] = EMPTY_VALUE;
            SMIBufferDb[i] = EMPTY_VALUE;
            colors[i] = colors[i+1];
               if (ColorOnSlope)
               {
                  if (SMIBuffer[i]>SMIBuffer[i+1]) colors[i]= 1;
                  if (SMIBuffer[i]<SMIBuffer[i+1]) colors[i]=-1;
               }
               else
               {
                  if (SMIBuffer[i]>SignalBuffer[i+1]) colors[i]= 1;
                  if (SMIBuffer[i]<SignalBuffer[i+1]) colors[i]=-1;
               }
               if (colors[i] == 1) SMIBufferDa[i] = SMIBuffer[i];
               if (colors[i] ==-1) SMIBufferDb[i] = SMIBuffer[i];
      }
   manageAlerts();
   return(0);
   }
   
   //
   //
   //
   //
   //

   limit = MathMax(limit,MathMin(Bars-1,iCustom(NULL,timeFrame,indicatorFileName,"returnBars",0,0)*timeFrame/Period()));
   for (i=limit; i>=0; i--)
   {
      int y = iBarShift(NULL,timeFrame,Time[i]);
         SMIBuffer[i]    = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",SMIPeriod1,SMIPeriod2,SMIPeriod3,SignalPeriod,2,y);
         SignalBuffer[i] = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",SMIPeriod1,SMIPeriod2,SMIPeriod3,SignalPeriod,3,y);
         colors[i]       = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",SMIPeriod1,SMIPeriod2,SMIPeriod3,SignalPeriod,4,y);
         SMIBufferDa[i]  = EMPTY_VALUE;
         SMIBufferDb[i]  = EMPTY_VALUE;
         colors[i] = colors[i+1];
            if (ColorOnSlope)
            {
               if (SMIBuffer[i]>SMIBuffer[i+1]) colors[i]= 1;
               if (SMIBuffer[i]<SMIBuffer[i+1]) colors[i]=-1;
            }
            else
            {
               if (SMIBuffer[i]>SignalBuffer[i+1]) colors[i]= 1;
               if (SMIBuffer[i]<SignalBuffer[i+1]) colors[i]=-1;
            }
            if (colors[i] == 1) SMIBufferDa[i] = SMIBuffer[i];
            if (colors[i] ==-1) SMIBufferDb[i] = SMIBuffer[i];
   }
manageAlerts();
return(0);
}   

//-------------------------------------------------------------------
//
//-------------------------------------------------------------------
//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

//
//
//
//
//

int stringToTimeFrame(string tfs)
{
   tfs = stringUpperCase(tfs);
   for (int i=ArraySize(iTfTable)-1; i>=0; i--)
         if (tfs==sTfTable[i] || tfs==""+iTfTable[i]) return(MathMax(iTfTable[i],Period()));
                                                      return(Period());
}
string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}

//
//
//
//
//

string stringUpperCase(string str)
{
   string   s = str;

   for (int length=StringLen(str)-1; length>=0; length--)
   {
      int tchar = StringGetChar(s, length);
         if((tchar > 96 && tchar < 123) || (tchar > 223 && tchar < 256))
                     s = StringSetChar(s, length, tchar - 32);
         else if(tchar > -33 && tchar < 0)
                     s = StringSetChar(s, length, tchar + 224);
   }
   return(s);
}

//
//
//
//
//

void manageAlerts()
{
   if (!calculateValue && alertsOn)
   {
      if (alertsOnCurrent)
           int whichBar = 0;
      else     whichBar = 1;  whichBar = iBarShift(NULL,0,iTime(NULL,timeFrame,whichBar));
      if (colors[whichBar] != colors[whichBar+1])
      {
         if (colors[whichBar] == 1) doAlert(whichBar,"up trend");
         if (colors[whichBar] ==-1) doAlert(whichBar,"down trend");
      }         
   }
}   

//
//
//
//
//

void doAlert(int forBar, string doWhat)
{
   static string   previousAlert="nothing";
   static datetime previousTime;
   string message;
   
   if (previousAlert != doWhat || previousTime != Time[forBar]) {
       previousAlert  = doWhat;
       previousTime   = Time[forBar];

       //
       //
       //
       //
       //

       message =  StringConcatenate(Symbol()," at ",TimeToStr(TimeLocal(),TIME_SECONDS)," - ",timeFrameToString(timeFrame)+" Blau - smi ",doWhat);
          if (alertsMessage) Alert(message);
          if (alertsNotify)  SendNotification(StringConcatenate(Symbol(), Period() ," Blau - smi " +" "+message));
          if (alertsEmail)   SendMail(StringConcatenate(Symbol()," Blau - smi "),message);
          if (alertsSound)   PlaySound(soundFile);
   }
}

